#!/bin/sh

DATECALLSPERCYCLE=5
TIMEBETWEENDATECALLS=2
MAXRSSROUNDS=5
MYPID=$$
RSSDISPLAYTIME=10
RSSFEEDS="http://slashdot.org/index.rss"  # separate urls by space
RSSFILE="$HOME/.wmii/.rssdata"

usage() {
    echo "usage: $0" >&2
    echo "GLOBALCONFDIR has to be set to wmii's global configuration " >&2
    echo "directory." >&2
    exit $1
}

getfile() {
    if [ -r "$HOME/.wmii/$1" ]; then
        echo "$HOME/.wmii/$1"
    elif [ -r "$GLOBALCONFDIR/$1" ]; then
        echo "$GLOBALCONFDIR/$1"
    else
        echo "error: $0: couldn't read $1" >&2
        usage 2
    fi
}

remove_own_pidfile() {
    [ -f "$STATUS_PIDFILE" ] && \
        [ "`cat \"$STATUS_PIDFILE\"`" = $MYPID ] && \
        rm "$STATUS_PIDFILE"
}

init() {
    # remove stale pidfiles:
    for file in `ls ${STATUS_PIDFILE_PREFIX}* 2>/dev/null`; do
        kill -s 0 `cat "$file"` >/dev/null 2>&1 || rm "$file"
    done

    if [ -f "$STATUS_PIDFILE" ]; then
        # another status.sh may still be running; try to kill it
        kill "`cat \"$STATUS_PIDFILE\"`" >/dev/null 2>&1
    fi

    echo $MYPID > "$STATUS_PIDFILE"

    trap "remove_own_pidfile; exit" TERM
}

rss_feeds() {
    # get rss feeds (write them to $RSSFILE) and print PID of wget
    wget -O "$RSSFILE" $RSSFEEDS >/dev/null 2>&1 &
    echo $!
}

rss_link() {
    # print link of nth rss entry where n is $1
    grep '^\W*<link>' "$RSSFILE" | grep '</link>' | \
        sed -n $1's/.*<link>\(.*\)<\/link>.*/\1/p'
}

rss_size() {
    # print number of rss entries
    if [ -f "$RSSFILE" ] && [ -r "$RSSFILE" ]; then
        ntitles="`grep '^\W*<title>' \"$RSSFILE\" | wc -l`"
        nlinks="`grep '^\W*<link>' \"$RSSFILE\" | wc -l`"

        if [ $ntitles -lt $nlinks ]; then
            echo $ntitles
        else
            echo $nlinks
        fi
    else
        echo 0
    fi
}

rss_title() {
    # print title of nth rss entry where n is $1
    grep '^\W*<title>' "$RSSFILE" | grep '</title>' | \
        sed -n $1's/.*<title>\(.*\)<\/title>.*/\1/p'
}

date_cycle() {
    j=1
    while [ $j -le $DATECALLSPERCYCLE ]; do
        $WRITE /bar/label/status/text "`date \"+%a %F\"` `uptime`" 2>/dev/null
        sleep $TIMEBETWEENDATECALLS
        j=`expr $j + 1`
    done
}

rss_cycle() {
    : ${rssentry:=1}
    : ${rssround:=$MAXRSSROUNDS}
    : ${wgetpid:=0}

    if [ "$rssentry" -gt `rss_size` ]; then
        # all entries have been displayed
        rssentry=1
        rssround=`expr $rssround + 1`
        if [ $rssround -gt $MAXRSSROUNDS ]; then
            # get new feeds
            rssround=1
            wgetpid=`rss_feeds`
        fi
    fi

    [ $wgetpid -ne 0 ] && kill -s 0 $wgetpid 2>/dev/null && \
        return  # wget is still running

    $WRITE /bar/label/status/text "`rss_title $rssentry`" 2>/dev/null
    $WRITE /bar/label/status/event/b1click \
        "\${BROWSER:-mozilla} '`rss_link $rssentry`'" 2>/dev/null

    rssentry=`expr $rssentry + 1`
    sleep $RSSDISPLAYTIME
}

#---------------------------------------------------------------------

[ $# -ne 0 ] && usage 1

. "`getfile rc.conf`";  # load variable definitions

init

while true; do
    if $READ /bar >/dev/null 2>&1; then
        date_cycle
        # uncomment to enable rss stuff:
        #rss_cycle
    else
        # exit if wmibar or wmifs aren't running
        remove_own_pidfile
        break
    fi
done
